from __future__ import print_function
from builtins import str
import os
import unittest
import shutil
import yaml
from neddy.utKit import utKit
from fundamentals import tools
from os.path import expanduser
home = expanduser("~")

packageDirectory = utKit("").get_project_root()
settingsFile = packageDirectory + "/test_settings.yaml"

su = tools(
    arguments={"settingsFile": settingsFile},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName=None,
    defaultSettingsFile=False
)
arguments, settings, log, dbConn = su.setup()

# SETUP PATHS TO COMMON DIRECTORIES FOR TEST DATA
moduleDirectory = os.path.dirname(__file__)
pathToInputDir = moduleDirectory + "/input/"
pathToOutputDir = moduleDirectory + "/output/"

try:
    shutil.rmtree(pathToOutputDir)
except:
    pass
# COPY INPUT TO OUTPUT DIR
shutil.copytree(pathToInputDir, pathToOutputDir)

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)

class test_conesearch(unittest.TestCase):

    def test_conesearch_function(self):
        from neddy import conesearch
        kwargs = {}
        kwargs["log"] = log
        kwargs["ra"] = 204.253833
        kwargs["dec"] = -29.865750
        kwargs["radiusArcsec"] = 15
        kwargs["nearestOnly"] = False
        kwargs["unclassified"] = True
        testObject = conesearch(**kwargs)
        testObject.get()

    def test_conesearch_function_02(self):
        from neddy import conesearch
        import codecs
        pathToReadFile = pathToInputDir + "/coordinates.txt"
        readFile = codecs.open(pathToReadFile, encoding='utf-8', mode='r')

        listOfCoordinates = []
        for line in readFile.readlines():
            line = line.strip()
            [ra, dec] = line.split()
            listOfCoordinates.append(str(ra) + " " + str(dec))

        kwargs = {}
        kwargs["log"] = log
        kwargs["nearestOnly"] = False
        kwargs["unclassified"] = True
        kwargs["radiusArcsec"] = 29
        kwargs["listOfCoordinates"] = listOfCoordinates
        testObject = conesearch(**kwargs)
        testObject.get()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



from __future__ import absolute_import
from . import cl_utils
from ._basesearch import _basesearch
from . import utKit
from . import cl_utils
from .namesearch import namesearch
from .conesearch import conesearch



#!/usr/local/bin/python
# encoding: utf-8
"""
*Perform a conesearch on NED*

:Author:
    David Young
"""
from __future__ import print_function
from __future__ import division
from future import standard_library
standard_library.install_aliases()
from past.utils import old_div
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import urllib.request, urllib.parse, urllib.error
from docopt import docopt
from fundamentals.download import multiobject_download
from fundamentals import tools, times
from neddy import _basesearch
from neddy import namesearch
# from ..__init__ import *

###################################################################
# CLASSES                                                         #
###################################################################
class conesearch(_basesearch):
    """
    *The worker class for the conesearch module*

    **Key Arguments**

    - ``log`` -- logger
    - ``ra`` -- ra
    - ``dec`` -- dec
    - ``radiusArcsec`` -- radiusArcsec
    - ``nearestOnly`` -- return only the nearest object from NED
    - ``unclassified`` -- include the unclassified sources in the return results
    - ``quiet`` -- don't print to stdout
    - ``listOfCoordinates`` -- list of coordinates ra dec radiusArcsec
    - ``outputFilePath`` -- path to output file
    - ``verbose`` -- return more metadata for matches
    

        - ``redshift`` -- redshift constraint

    .. todo::

        - @review: when complete, clean conesearch class
        - @review: when complete add logging
        - @review: when complete, decide whether to abstract class to another module
    """
    # Initialisation

    def __init__(
        self,
        log,
        ra=False,
        dec=False,
        radiusArcsec=False,
        nearestOnly=False,
        unclassified=False,
        quiet=False,
        listOfCoordinates=False,
        outputFilePath=False,
        verbose=False,
        redshift=False
    ):
        self.log = log
        self.log.debug("instansiating a new 'conesearch' object")
        self.ra = ra
        self.dec = dec
        self.arcsec = radiusArcsec
        self.nearestOnly = nearestOnly
        self.unclassified = unclassified
        self.quiet = quiet
        self.listOfCoordinates = listOfCoordinates
        self.outputFilePath = outputFilePath
        self.verbose = verbose
        self.redshift = redshift

        # xt-self-arg-tmpx

        # VARIABLE DATA ATRRIBUTES
        self.arcmin = old_div(float(self.arcsec), 60.)
        self.resultSpacing = 30

        # Initial Actions
        # CREATE A LIST IF SINGLE COORDINATES GIVEN
        if self.listOfCoordinates == False:
            self.listOfCoordinates = ["%(ra)s %(dec)s" % locals()]
        self._convert_coordinates_to_decimal_degrees()

        return None

    # METHOD ATTRIBUTES
    def get(self):
        """
        *get the conesearch object*

        **Return**

        - ``conesearch``
        

        .. todo::

            - @review: when complete, clean get method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``get`` method')

        # SEARCH NED WITH SINGLE CONESEARCHES TO RETURN LIST OF MATCHED NAMES
        names, searchParams = self.get_crossmatch_names(
            listOfCoordinates=self.listOfCoordinates,
            radiusArcsec=self.arcsec
        )

        # NOW PERFORM A NAME SEARCH AGAINST THE MATCHED NAMES
        search = namesearch(
            log=self.log,
            names=names,
            quiet=False,
            searchParams=searchParams,
            verbose=self.verbose,
            outputFilePath=self.outputFilePath
        )
        search.get()

        self.log.debug('completed the ``get`` method')
        return conesearch

    # use the tab-trigger below for new method
    def _get_ned_query_url(
            self,
            raDeg,
            decDeg,
            arcsec):
        """
        *single ned conesearch*

        **Key Arguments**

        # -
        

        **Return**

        - None
        

        .. todo::

            - @review: when complete, clean _get_ned_query_url method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_get_ned_query_url`` method')

        radArcMin = old_div(float(arcsec), (60.))

        if self.redshift == True:
            z_constraint = "Available"
        else:
            z_constraint = "Unconstrained"

        url = "http://ned.ipac.caltech.edu/cgi-bin/objsearch"
        params = {
            "in_csys": "Equatorial",
            "in_equinox": "J2000.0",
            "lon": "%(raDeg)0.6fd" % locals(),
            "lat": "%(decDeg)0.6fd" % locals(),
            "radius": "%(radArcMin)0.6s" % locals(),
            "hconst": "73",
            "omegam": "0.27",
            "omegav": "0.73",
            "corr_z": "1",
            "z_constraint": z_constraint,
            "z_value1": "",
            "z_value2": "",
            "z_unit": "z",
            "ot_include": "ANY",
            "nmp_op": "ANY",
            "out_csys": "Equatorial",
            "out_equinox": "J2000.0",
            "obj_sort": "Distance to search center",
            "of": "ascii_bar",
            "zv_breaker": "30000.0",
            "list_limit": "500",
            "img_stamp": "NO",
            "search_type": "Near Position Search",

        }

        url = url + "?" + urllib.parse.urlencode(params)
        if not self.unclassified:
            url = url + "&" + urllib.parse.urlencode({"ot_include": "ANY"})
            in_objtypes1 = ["Galaxies", "GPairs", "GTriples", "GGroups",
                            "GClusters", "QSO", "QSOGroups", "GravLens", "AbsLineSys", "EmissnLine"]
            for o in in_objtypes1:
                url = url + "&" + urllib.parse.urlencode({"in_objtypes1": o})
            in_objtypes3 = ["Supernovae", "HIIregion", "PN", "SNR", "StarAssoc", "StarClust", "MolCloud", "Nova", "VarStar", "WolfRayet",
                            "CarbonStar", "PofG", "Other", "Star", "BlueStar", "RedStar", "Pulsar", "ReflNeb", "DblStar", "EmissnObj", "EmissnNeb", "WhiteDwarf"]
            for o in in_objtypes3:
                url = url + "&" + urllib.parse.urlencode({"in_objtypes3": o})

        self.log.debug('completed the ``_get_ned_query_url`` method')
        return url

    # use the tab-trigger below for new method
    def get_crossmatch_names(
            self,
            listOfCoordinates=False,
            radiusArcsec=False):
        """
        *get corssmatch names*

        **Key Arguments**

        - ``listOfCoordinates`` -- list of the coordinates to conesearch
        - ``radiusArcsec`` -- the search radius
        

        **Return**

        - None
        

        .. todo::

            - @review: when complete, clean get_crossmatch_names method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``get_crossmatch_names`` method')

        if listOfCoordinates == False:
            listOfCoordinates = self.listOfCoordinates
        if radiusArcsec == False:
            radiusArcsec = self.arcsec

        names = []
        searchParams = []
        nedUrls = []
        for i, coord in enumerate(listOfCoordinates):
            url = self._get_ned_query_url(
                raDeg=coord[0],
                decDeg=coord[1],
                arcsec=radiusArcsec
            )
            nedUrls.append(url)

        count = len(nedUrls)
        if count:
            print("%(count)s NED conesearch URLs have been built. Requesting from NED ..." % locals())

        localUrls = multiobject_download(
            urlList=nedUrls,
            downloadDirectory="/tmp",
            log=self.log,
            timeStamp=True,
            timeout=3600,
            concurrentDownloads=10,
            resetFilename=False,
            credentials=False,  # { 'username' : "...", "password", "..." }
            longTime=True,
            indexFilenames=True
        )

        count = len(localUrls)
        if count:
            print("%(count)s conesearch results downloaded from NED" % locals())

        for ii, nedResults in enumerate(localUrls):
            if nedResults == None:
                thisUrl = nedUrls[ii]
                self.log.error(
                    'cound not download results for NED URL: %(thisUrl)s' % locals())
                continue
            i = int(nedResults.split("/")[-1].split("_")[0])
            results, resultLen = self._parse_the_ned_position_results(
                ra=listOfCoordinates[i][0],
                dec=listOfCoordinates[i][1],
                nedResults=nedResults
            )
            print("  %(resultLen)s returned from single NED conesearch" % locals())
            if resultLen > 45000:
                print(" To many results returned from single NED query ... aborting!")
                subnames, subsearchParams = self._oversized_subqueries(
                    coordinate=listOfCoordinates[i],
                    radiusArcsec=radiusArcsec
                )
                names += subnames
                searchParams += subsearchParams
            else:
                for r in results:
                    searchParams.append(
                        {"searchIndex": i + 1, "searchRa": r["searchRa"], "searchDec": r["searchDec"]})
                    names.append(r["matchName"])
            os.remove(nedResults)

        self.log.debug('completed the ``get_crossmatch_names`` method')
        return names, searchParams

    # use the tab-trigger below for new method
    def _oversized_subqueries(
            self,
            coordinate,
            radiusArcsec):
        """
        *subdivide oversized query*

        **Key Arguments**

        # -
        

        **Return**

        - None
        

        .. todo::

            - @review: when complete, clean _oversized_subqueries method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_oversized_subqueries`` method')

        import math

        smallerRadiusArcsec = old_div(radiusArcsec, 2.)
        print("Calculating 7 sub-disks for coordinates %(coordinate)s, with smaller search radius of %(smallerRadiusArcsec)s arcsec" % locals())

        ra = coordinate[0]
        dec = coordinate[1]

        shifts = [
            (0, 0),
            (0, old_div(math.sqrt(3.), 2.)),
            (old_div(3., 4.), old_div(math.sqrt(3.), 4.)),
            (old_div(3., 4.), old_div(-math.sqrt(3.), 4.)),
            (0, old_div(-math.sqrt(3.), 2.)),
            (old_div(-3., 4.), old_div(-math.sqrt(3.), 4.)),
            (old_div(-3., 4.), old_div(math.sqrt(3.), 4.))
        ]

        subDiskCoordinates = []
        count = 0
        for s in shifts:
            x1 = ra + s[0] * radiusArcsec / (60 * 60)
            y1 = dec + s[1] * radiusArcsec / (60 * 60)
            subDiskCoordinates.append((x1, y1))

        names, searchParams = self.get_crossmatch_names(
            listOfCoordinates=subDiskCoordinates,
            radiusArcsec=smallerRadiusArcsec
        )

        self.log.debug('completed the ``_oversized_subqueries`` method')
        return names, searchParams

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*The base class for NED searches*

:Author:
    David Young

:Date Created:
    May 6, 2015
"""
from __future__ import print_function
from __future__ import division
################# GLOBAL IMPORTS ####################
from builtins import zip
from builtins import str
from builtins import range
from builtins import object
from past.utils import old_div
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import codecs
import re
import string
import csv
from docopt import docopt
from astrocalc.coords import unit_conversion
from fundamentals import tools, times


class _basesearch(object):

    """
    The base-class for searching NED
    """
    # Initialisation

    def __init__(
        self
    ):
        return None

    def _convert_coordinates_to_decimal_degrees(
            self):
        """
        *convert coordinates to decimal degrees*
        """
        self.log.debug(
            'completed the ````_convert_coordinates_to_decimal_degrees`` method')

        converter = unit_conversion(
            log=self.log
        )

        # CONVERT ALL COORDINATES TO DECIMAL DEGREES
        if len(self.listOfCoordinates) and isinstance(self.listOfCoordinates[0], ("".__class__, u"".__class__)):
            sources = []
            sources[:] = [s.split(" ") for s in self.listOfCoordinates]

        else:
            sources = self.listOfCoordinates
        self.listOfCoordinates = []

        # TEST IF CONVERSION IS REQUIRED
        try:
            ra = float(sources[0][0])
            convert = False
        except:
            convert = True

        if convert == True:
            raDegs = []
            raDegs[:] = [converter.ra_sexegesimal_to_decimal(
                ra=s[0]) for s in sources]
            decDegs = []
            decDegs[:] = [converter.dec_sexegesimal_to_decimal(
                dec=s[1]) for s in sources]
            self.listOfCoordinates = []
            self.listOfCoordinates[:] = [[raDeg, decDeg]
                                         for raDeg, decDeg in zip(raDegs, decDegs)]

        else:
            self.listOfCoordinates = []
            self.listOfCoordinates[:] = [[float(s[0]), float(s[1])]
                                         for s in sources]

        self.log.debug(
            'completed the ``_convert_coordinates_to_decimal_degrees`` method')
        return None

    def _parse_the_ned_position_results(
            self,
            ra,
            dec,
            nedResults):
        """
        *parse the ned results*

        **Key Arguments:**
            - ``ra`` -- the search ra
            - ``dec`` -- the search dec

        **Return:**
            - ``results`` -- list of result dictionaries
        """
        self.log.debug('starting the ``_parse_the_ned_results`` method')

        results = []
        resultLen = 0
        if nedResults:
            # OPEN THE RESULT FILE FROM NED
            pathToReadFile = nedResults
            try:
                self.log.debug("attempting to open the file %s" %
                               (pathToReadFile,))
                readFile = codecs.open(
                    pathToReadFile, encoding='utf-8', mode='rb')
                thisData = readFile.read()
                readFile.close()
            except IOError as e:
                message = 'could not open the file %s' % (pathToReadFile,)
                self.log.critical(message)
                raise IOError(message)
            readFile.close()

            # CHECK FOR ERRORS
            if "Results from query to  NASA/IPAC Extragalactic Database" not in thisData:
                print("something went wrong with the NED query")
                self.log.error(
                    "something went wrong with the NED query" % locals())
                sys.exit(0)

            # SEARCH FROM MATCHES IN RESULTS FILE
            matchObject = re.search(
                r"No\.\|Object Name.*?\n(.*)", thisData, re.S)
            if matchObject:
                try:
                    theseLines = str.split(matchObject.group(), '\n')
                except:
                    theseLines = string.split(matchObject.group(), '\n')
                resultLen = len(theseLines)
                csvReader = csv.DictReader(
                    theseLines, dialect='excel', delimiter='|', quotechar='"')
                for row in csvReader:
                    thisEntry = {"searchRa": ra, "searchDec": dec,
                                 "matchName": row["Object Name"].strip()}
                    results.append(thisEntry)
                    if self.nearestOnly:
                        break

        self.log.debug('completed the ``_parse_the_ned_results`` method')
        return results, resultLen

    # use the tab-trigger below for new method
    def _parse_the_ned_object_results(
            self):
        """
        *parse the ned results*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _parse_the_ned_results method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_parse_the_ned_results`` method')

        results = []
        headers = ["objectName", "objectType", "raDeg", "decDeg",
                   "redshift", "redshiftFlag"]
        if self.nedResults:
            pathToReadFile = self.nedResults
            try:
                self.log.debug("attempting to open the file %s" %
                               (pathToReadFile,))
                readFile = codecs.open(
                    pathToReadFile, encoding='utf-8', mode='rb')
                thisData = readFile.read()
                readFile.close()
            except IOError as e:
                message = 'could not open the file %s' % (pathToReadFile,)
                self.log.critical(message)
                raise IOError(message)
            readFile.close()

            matchObject = re.search(
                r"No\.\|Object Name.*?\n(.*)", thisData, re.S)
            if matchObject:
                # Print the header for stdout
                thisHeader = "| "
                for head in headers:
                    thisHeader += str(head).ljust(self.resultSpacing,
                                                  ' ') + " | "
                if not self.quiet:
                    print(thisHeader)
                try:
                    theseLines = str.split(matchObject.group(), '\n')
                except:
                    theseLines = string.split(matchObject.group(), '\n')
                csvReader = csv.DictReader(
                    theseLines, dialect='excel', delimiter='|', quotechar='"')
                for row in csvReader:
                    thisDict = {}
                    thisRow = "| "
                    thisDict["raDeg"] = row["RA(deg)"].strip()
                    thisDict["decDeg"] = row["DEC(deg)"].strip()
                    thisDict["redshift"] = row["Redshift"].strip()
                    thisDict["redshiftFlag"] = row["Redshift Flag"].strip()
                    thisDict["objectName"] = row["Object Name"].strip()
                    thisDict["objectType"] = row["Type"].strip()

                    results.append(thisDict)
                    for head in headers:
                        thisRow += str(thisDict[head]
                                       ).ljust(self.resultSpacing, ' ') + " | "
                    if not self.quiet:
                        print(thisRow)

            else:
                for head in headers:
                    thisRow += str("").ljust(self.resultSpacing, ' ') + " | "
                if not self.quiet:
                    print(thisRow)
        else:
            # Print the header for stdout
            thisHeader = "| "
            for head in headers:
                thisHeader += str(head).ljust(self.resultSpacing, ' ') + " | "
            if not self.quiet:
                print(thisHeader)
            thisRow = "| "
            for head in headers:
                thisRow += str("").ljust(self.resultSpacing, ' ') + " | "
            if not self.quiet:
                print(thisRow)

        self.log.debug('completed the ``_parse_the_ned_results`` method')
        return results

        # use the tab-trigger below for new method
    def _convert_html_to_csv(
            self):
        """
        *contert html to csv*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _convert_html_to_csv method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_convert_html_to_csv`` method')

        import codecs
        allData = ""
        regex1 = re.compile(
            r'.*<PRE><strong>   (.*?)</strong>(.*?)</PRE></TABLE>.*', re.I | re.S)
        regex2 = re.compile(r'\|(\w)\|', re.I | re.S)
        for thisFile in self.nedResults:
            pathToReadFile = thisFile
            try:
                self.log.debug("attempting to open the file %s" %
                               (pathToReadFile,))
                readFile = codecs.open(
                    pathToReadFile, encoding='utf-8', mode='r')
                thisData = readFile.read()
                readFile.close()
            except IOError as e:
                message = 'could not open the file %s' % (pathToReadFile,)
                self.log.critical(message)
                raise IOError(message)
            except:
                if pathToReadFile == None:
                    message = 'we have no file to open'
                    self.log.error(message)
                    continue
            readFile.close()

            self.log.debug("regex 1 - sub")
            thisData = regex1.sub("\g<1>\g<2>", thisData)
            self.log.debug("regex 2 - sub")
            thisData = regex2.sub("abs(\g<1>)", thisData)
            self.log.debug("replace text")
            thisData = thisData.replace("|b|", "abs(b)")

            writeFile = codecs.open(pathToReadFile, encoding='utf-8', mode='w')
            writeFile.write(thisData)
            writeFile.close()

        self.log.debug('completed the ``_convert_html_to_csv`` method')
        return None

    # use the tab-trigger below for new method
    def _parse_the_ned_list_results(
            self):
        """
        *parse the ned results*

        **Key Arguments:**
            # -

        **Return:**
            - None

        .. todo::

            - @review: when complete, clean _parse_the_ned_results method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_parse_the_ned_list_results`` method')

        results = []

        # CHOOSE VALUES TO RETURN
        allHeaders = ["searchIndex", "searchRa", "searchDec", "row_number", "input_note", "input_name", "ned_notes", "ned_name", "ra", "dec", "eb-v", "object_type", "redshift", "redshift_err", "redshift_quality", "magnitude_filter",
                      "major_diameter_arcmin", "minor_diameter_arcmin", "morphology", "hierarchy", "galaxy_morphology", "radio_morphology", "activity_type", "distance_indicator", "distance_mod", "distance"]
        if self.verbose == True:
            headers = ["searchIndex", "searchRa", "searchDec", "row_number", "input_note", "input_name", "ned_notes", "ned_name", "ra", "dec", "eb-v", "object_type", "redshift", "redshift_err", "redshift_quality", "magnitude_filter",
                       "major_diameter_arcmin", "minor_diameter_arcmin", "morphology", "hierarchy", "galaxy_morphology", "radio_morphology", "activity_type", "distance_indicator", "distance_mod", "distance"]
        else:
            headers = [
                "searchIndex", "searchRa", "searchDec", "ned_name", "ra", "dec", "object_type", "redshift"]

        if self.theseBatchParams == False:
            allHeaders = allHeaders[3:]
            headers = headers[3:]

        for thisFile in self.nedResults:
            if thisFile:
                pathToReadFile = thisFile
                # FIND THE BATCH INDEX NUMBER
                thisIndex = int(thisFile.split("/")[-1].split("_")[0])
                try:
                    self.log.debug("attempting to open the file %s" %
                                   (pathToReadFile,))
                    readFile = codecs.open(
                        pathToReadFile, encoding='utf-8', mode='rb')
                    thisData = readFile.read()
                    readFile.close()
                except IOError as e:
                    message = 'could not open the file %s' % (pathToReadFile,)
                    self.log.critical(message)
                    raise IOError(message)
                readFile.close()

                # GRAB THE ROWS OF DATA
                matchObject = re.search(
                    r"\n1\s*?\|\s*?.*", thisData, re.S)
                thisRow = ""
                if matchObject:
                    thisHeader = ""
                    for head in allHeaders:
                        thisHeader += str(head).ljust(self.resultSpacing,
                                                      ' ') + " | "
                    try:
                        theseLines = str.split(matchObject.group(), '\n')[1:]
                    except:
                        theseLines = string.split(
                            matchObject.group(), '\n')[1:]

                    if self.theseBatchParams:
                        newLines = []
                        for t, b in zip(theseLines, self.theseBatchParams[thisIndex]):
                            t = "%s | %s | %s | %s " % (
                                b["searchIndex"], b["searchRa"], b["searchDec"], t)
                            newLines.append(t)
                        theseLines = newLines

                    theseLines = [thisHeader] + theseLines
                    csvReader = csv.DictReader(
                        theseLines, dialect='excel', delimiter='|', quotechar='"')
                    for row in csvReader:
                        thisDict = {}
                        row = dict(row)
                        if not list(row.keys()):
                            continue
                        if None in list(row.keys()):
                            continue
                        if "ned_name" not in ("").join(list(row.keys())).lower():
                            continue
                        for k, v in list(row.items()):
                            try:
                                # self.log.debug("attempting to strip ned key")
                                k = k.strip()
                            except Exception as e:
                                self.log.error(
                                    'cound not strip ned key (%(k)s, %(v)s)' % locals())
                                self.log.error(
                                    "could not strip ned key - failed with this error: %s " % (str(e),))
                                break
                            if (k == "ra" or k == "dec"):
                                v = v.replace("h", ":").replace(
                                    "m", ":").replace("d", ":").replace("s", "")
                            if isinstance(v, ("".__class__, u"".__class__)):
                                v = v.strip()
                            thisDict[k] = v
                        results.append(thisDict)

                os.remove(thisFile)

        self.log.debug('completed the ``_parse_the_ned_list_results`` method')
        return results, headers

    def _split_incoming_queries_into_batches(
            self,
            sources,
            searchParams=False):
        """
        *split incoming queries into batches*

        **Key Arguments:**
            - ``sources`` -- sources to split into batches
            - ``searchParams`` -- search params associated with batches

        **Return:**
            - ``theseBatches`` -- list of batches
            - ``theseBatchParams`` -- params associated with batches
        """
        self.log.debug(
            'completed the ````_split_incoming_queries_into_batches`` method')

        batchSize = 180
        total = len(sources)
        batches = int(old_div(total, batchSize)) + 1

        start = 0
        end = 0
        theseBatches = []
        theseBatchParams = []
        for i in range(batches):
            end = end + batchSize
            start = i * batchSize
            thisBatch = sources[start:end]
            theseBatches.append(thisBatch)

            if searchParams != False:
                thisBatch = searchParams[start:end]
                theseBatchParams.append(thisBatch)

        if len(theseBatchParams) == 0:
            theseBatchParams = False

        self.log.debug(
            'completed the ``_split_incoming_queries_into_batches`` method')
        return theseBatches, theseBatchParams

        # use the tab-trigger below for new method
        # xt-class-method



#!/usr/bin/env python
# encoding: utf-8
"""
Documentation for neddy can be found here: http://neddy.readthedocs.org

Usage:
    neddy [-nuvr] cone (filelist <pathToCoordinateList> <radiusArcsec> | <ra> <dec> <radiusArcsec>) [<outPutFile>]
    neddy [-v] obj <objectName> [<objectName>...]
    
    -h, --help            show this help message
    -n, --nearest         nearest object only
    -u, --unclassified    include unclassifed extra-galaxtic objects
    -v, --verbose         return more metadata for matches
    -r, --redshift        redshift must be available
    ra                    ra (decimal degrees or sexegesimal)
    dec                   dec (decimal degrees or sexegesimal)
    radiusArcsec          radiusArcsec (conesearch radius)
    objectName            objectName (the name of the object)
    pathToCoordinateList  path to list of ra dec radiusArcsec
    outPutFile            path to outputfile
"""
from builtins import str
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from fundamentals import tools, times
from subprocess import Popen, PIPE, STDOUT

def tab_complete(text, state):
    return (glob.glob(text + '*') + [None])[state]

def main(arguments=None):
    """
    *The main function used when `cl_utils.py` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="WARNING",
        options_first=False,
        projectName="neddy",
        defaultSettingsFile=True
    )
    arguments, settings, log, dbConn = su.setup()

    # tab completion for raw_input
    readline.set_completer_delims(' \t\n;')
    readline.parse_and_bind("tab: complete")
    readline.set_completer(tab_complete)

    # UNPACK REMAINING CL ARGUMENTS USING `EXEC` TO SETUP THE VARIABLE NAMES
    # AUTOMATICALLY
    a = {}
    for arg, val in list(arguments.items()):
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        a[varname] = val
        if arg == "--dbConn":
            dbConn = val
            a["dbConn"] = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE cl_utils.py AT %s' %
        (startTime,))

    # set options interactively if user requests
    if "interactiveFlag" in a and a["interactiveFlag"]:

        # load previous settings
        moduleDirectory = os.path.dirname(__file__) + "/resources"
        pathToPickleFile = "%(moduleDirectory)s/previousSettings.p" % locals()
        try:
            with open(pathToPickleFile):
                pass
            previousSettingsExist = True
        except:
            previousSettingsExist = False
        previousSettings = {}
        if previousSettingsExist:
            previousSettings = pickle.load(open(pathToPickleFile, "rb"))

        # x-raw-input
        # x-boolean-raw-input
        # x-raw-input-with-default-value-from-previous-settings

        # save the most recently used requests
        pickleMeObjects = []
        pickleMe = {}
        theseLocals = locals()
        for k in pickleMeObjects:
            pickleMe[k] = theseLocals[k]
        pickle.dump(pickleMe, open(pathToPickleFile, "wb"))

    if a["init"]:
        from os.path import expanduser
        home = expanduser("~")
        filepath = home + "/.config/neddy/neddy.yaml"
        try:
            cmd = """open %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        try:
            cmd = """start %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        return

    ra = a["ra"]
    dec = a["dec"]
    radiusArcsec = a["radiusArcsec"]
    objectName = a["objectName"]
    pathToCoordinateList = a["pathToCoordinateList"]
    outPutFile = a["outPutFile"]
    nearest = a["nearest"]
    unclassified = a["unclassified"]
    redshift = a["redshift"]

    # CALL FUNCTIONS/OBJECTS
    if cone and filelist:
        import codecs
        pathToReadFile = pathToCoordinateList
        readFile = codecs.open(pathToReadFile, encoding='utf-8', mode='r')

        listOfCoordinates = []
        for line in readFile.readlines():
            line = line.strip()
            [ra, dec] = line.split()
            listOfCoordinates.append(str(ra) + " " + str(dec))
        search = conesearch(
            log=log,
            radiusArcsec=radiusArcsec,
            nearestOnly=nearestFlag,
            unclassified=unclassifiedFlag,
            listOfCoordinates=listOfCoordinates,
            outputFilePath=outPutFile,
            verbose=verboseFlag,
            redshift=redshiftFlag)
    elif cone:
        search = conesearch(
            log=log,
            ra=ra,
            dec=dec,
            radiusArcsec=radiusArcsec,
            nearestOnly=nearestFlag,
            unclassified=unclassifiedFlag,
            outputFilePath=outPutFile,
            verbose=verboseFlag,
            redshift=redshiftFlag
        )
    elif obj:
        search = namesearch(
            log=log,
            names=objectName,
            verbose=verboseFlag,
            outputFilePath=outPutFile
        )
    search.get()

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE cl_utils.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*A Name Searcher for NED*

:Author:
    David Young
"""
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
from builtins import str
import sys
import os
import readline
import glob
import pickle
import urllib.request, urllib.parse, urllib.error
from docopt import docopt
from fundamentals.download import multiobject_download
from fundamentals import tools, times
from neddy import _basesearch

###################################################################
# CLASSES                                                         #
###################################################################

class namesearch(_basesearch):
    """
    *The worker class for the namesearch module*

    **Key Arguments**

    - ``log`` -- logger
    - ``name`` -- name
    - ``quiet`` -- don't print to stdout
    - ``verbose`` -- return more metadata for matches
    - ``searchParams`` -- list of dictionaries to prepend to results
    - ``outputFilePath`` -- path to file to output results to
    
    """
    # Initialisation

    def __init__(
            self,
            log,
            names,
            quiet=False,
            verbose=False,
            searchParams=False,
            outputFilePath=False
    ):
        self.log = log
        log.debug("instantiating a new 'namesearch' object")
        self.names = names
        self.quiet = quiet
        self.verbose = verbose
        self.searchParams = searchParams
        self.outputFilePath = outputFilePath
        # xt-self-arg-tmpx

        # VARIABLE DATA ATRRIBUTES
        self.resultSpacing = 15

        # Initial Actions
        # CREATE A LIST IF SINGLE NAME GIVEN
        os.environ['TERM'] = 'vt100'
        if not isinstance(self.names, list):
            self.uplist = [self.names]
        else:
            self.uplist = self.names

        return None

    # METHOD ATTRIBUTES
    def get(self):
        """
        *get the namesearch object*

        **Return**

        - ``results``
        
        """
        self.log.debug('starting the ``get`` method')

        # SPLIT THE LIST OF NAMES INTO BATCHES
        self.theseBatches, self.theseBatchParams = self._split_incoming_queries_into_batches(
            sources=self.uplist,
            searchParams=self.searchParams
        )

        # PERFORM NAME QUERIES AGAINST NED
        self._build_api_url_and_download_results()
        self.results, self.headers = self._parse_the_ned_list_results()
        self._output_results()

        self.log.debug('completed the ``get`` method')
        return self.results

    def _build_api_url_and_download_results(
            self):
        """
        *build api url for NED to perform batch name queries*

        **Key Arguments**

        # -
        

        **Return**

        - None
        

        .. todo::

            - @review: when complete, clean _build_api_url_and_download_results method
            - @review: when complete add logging
        """
        self.log.debug(
            'completed the ````_build_api_url_and_download_results`` method')

        baseUrl = "https://ned.ipac.caltech.edu/cgi-bin/"
        command = "gmd"
        urlParameters = {
            "delimiter": "bar",
            "NO_LINKS": "1",
            "nondb": ["row_count", "user_name_msg", "user_objname"],
            "crosid": "objname",
            "enotes": "objnote",
            "position": ["ra,dec", "bhextin", "pretype", "z", "zunc", "zflag"],
            "gadata": ["magnit", "sizemaj", "sizemin", "morphol"],
            "attdat_CON": ["M", "S", "H", "R", "z"],
            "distance_CON": ["mm", "dmpc"],
            "attdat": "attned"
        }

        queryBase = "%(baseUrl)s%(command)s?uplist=" % locals()
        queryList = []

        # BUILD THE LIST OF QUERIES
        for batch in self.theseBatches:
            thisLength = len(batch)
            queryUrl = queryBase
            # ADD NAMES
            for thisIndex, thisName in enumerate(batch):
                queryUrl = queryUrl + urllib.parse.quote(thisName)
                if thisIndex < thisLength - 1:
                    queryUrl = queryUrl + "%0D"
            # ADD PARAMETERS
            for k, v in list(urlParameters.items()):
                if isinstance(v, list):
                    for item in v:
                        queryUrl = queryUrl + "&" + \
                            k + "=" + urllib.parse.quote(item)
                else:
                    queryUrl = queryUrl + "&" + k + "=" + urllib.parse.quote(v)
            queryList.append(queryUrl)

        # PULL THE RESULT PAGES FROM NED
        self.nedResults = multiobject_download(
            urlList=queryList,
            downloadDirectory="/tmp",
            log=self.log,
            timeStamp=1,
            timeout=3600,
            concurrentDownloads=10,
            resetFilename=False,
            credentials=False,  # { 'username' : "...", "password", "..." }
            longTime=True,
            indexFilenames=True
        )

        for thisIndex, r in enumerate(self.nedResults):
            if r == None:
                thisUrl = queryList[thisIndex]
                self.log.error(
                    'cound not download NED results for URL %(thisUrl)s' % locals())
                sys.exit(0)

        self._convert_html_to_csv()

        self.log.debug(
            'completed the ``_build_api_url_and_download_results`` method')
        return None

    def _output_results(
            self):
        """
        *output results*

        **Key Arguments**

        # -
        

        **Return**

        - None
        

        .. todo::

            - @review: when complete, clean _output_results method
            - @review: when complete add logging
        """
        self.log.debug('starting the ``_output_results`` method')

        content = ""
        maxNameLen = 0

        for r in self.results:
            if maxNameLen < len(r["ned_name"]):
                maxNameLen = len(r["ned_name"])

        if len(self.results) == 0:
            content += "No resuls found"
        else:
            thisHeader = "| "
            thisLine = "| "
            for head in self.headers:
                if head == "ned_name":
                    s = maxNameLen
                else:
                    s = self.resultSpacing
                thisHeader += str(head).ljust(s,
                                              ' ') + " | "
                thisLine += ":".ljust(s,
                                      '-') + " | "
            content += thisHeader
            content += "\n" + thisLine
            for r in self.results:
                thisRow = "| "
                for head in self.headers:
                    if head == "ned_name":
                        s = maxNameLen
                    else:
                        s = self.resultSpacing
                    thisRow += str(r[head]).ljust(s,
                                                  ' ') + " | "
                content += "\n" + thisRow

        if self.quiet == False:
            print(content)
        if self.outputFilePath:
            import codecs
            writeFile = codecs.open(
                self.outputFilePath, encoding='utf-8', mode='w')
            writeFile.write(content)
            writeFile.close()

        self.log.debug('completed the ``_output_results`` method')
        return None

    # use the tab-trigger below for new method
    # xt-class-method

    # 5. @flagged: what actions of the base class(es) need ammending? ammend them here
    # Override Method Attributes
    # method-override-tmpx
